package com.iamberry.wechat.handles.pay;

import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.StringReader;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.iamberry.wechat.core.entity.ResultMsg;
import com.iamberry.wechat.core.entity.coupon.CouponItemDto;
import com.iamberry.wechat.core.entity.coupon.CouponType;
import com.iamberry.wechat.core.entity.eo.Promotions;
import com.iamberry.wechat.core.entity.ism.IntegralConfig;
import com.iamberry.wechat.core.entity.ism.IntegralLog;
import com.iamberry.wechat.core.entity.order.OrderItem;
import com.iamberry.wechat.face.coupon.CouponTypeService;
import com.iamberry.wechat.face.coupon.WelfareService;
import com.iamberry.wechat.face.ism.IntegralLogService;
import com.iamberry.wechat.service.ActivityUtil;
import com.iamberry.wechat.tools.ResultInfo;
import com.iamberry.wechat.tools.SendMessageUtil;
import com.iamberry.wechat.tools.StrUtils;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.xml.sax.InputSource;

import com.iamberry.app.tool.log.RatFWLogger;
import com.iamberry.wechat.core.entity.coupon.CouponItem;
import com.iamberry.wechat.core.entity.drp.PlaceInfo;
import com.iamberry.wechat.core.entity.drp.PlaceOrder;
import com.iamberry.wechat.core.entity.member.CashLog;
import com.iamberry.wechat.core.entity.member.Member;
import com.iamberry.wechat.core.entity.mq.MQMessage;
import com.iamberry.wechat.core.entity.order.Order;
import com.iamberry.wechat.core.entity.wx.WxPayResult;
import com.iamberry.wechat.face.cart.CartService;
import com.iamberry.wechat.face.coupon.CouponItemService;
import com.iamberry.wechat.face.drp.PlaceInfoService;
import com.iamberry.wechat.face.drp.PlaceOrderService;
import com.iamberry.wechat.face.member.CashLogService;
import com.iamberry.wechat.face.member.MemberService;
import com.iamberry.wechat.face.order.AdminOrderService;
import com.iamberry.wechat.face.qrcode.QrcodeService;
import com.iamberry.wechat.face.reback.RebackServices;
import com.iamberry.wechat.handles.mq.MQServiceProxy;
import com.iamberry.wechat.tools.StaticInfo;

/**
 * @author 何秀刚
 * Class Description: 微信支付回调handler,所有的支付回调，都在本类完成
 * Create Date:2016年4月19日
 * Update Date:2016年4月19日
 */
@Controller
@RequestMapping("/callback")
public class ResponseWechatPayHandler {

	@Autowired
	private RatFWLogger ratFWLogger;
	
	private final Object lock = new Object();
	@Autowired
	private CartService cartService;
	@Autowired
	private PlaceOrderService placeOrderService;
	@Autowired
	private SendMessageUtil sendMessageUtil;
	@Autowired
	private AdminOrderService adminOrderService;
	@Autowired
	private MQServiceProxy mQservice;
	@Autowired
	private WelfareService welfareService;
	@Autowired
	CashLogService cashLogService;
	@Autowired
	CouponItemService couponItemService;
	@Autowired
	private IntegralLogService integralLogService;
	@Autowired
	private ActivityUtil activityUtil;
	@Autowired
	private MemberService memberService;
	@Autowired
	private CouponTypeService couponTypeService;

	/**
	 * 销售订单支付回调方法
	 */
	@SuppressWarnings("rawtypes")
	@RequestMapping("/orderPayBack")
	public void orderPayBack(HttpServletRequest request, 
			HttpServletResponse response) throws IOException {
		synchronized (lock) {
			String inputLine;
			StringBuilder notityXml = new StringBuilder();
			String resXml = "<xml><return_code><![CDATA[SUCCESS]]></return_code><return_msg><![CDATA[OK]]></return_msg></xml>";
			try {
				//获取XML参数
				while ((inputLine = request.getReader().readLine()) != null) {
					notityXml.append(inputLine);
				}
				request.getReader().close();
				// 解析XML格式的数据
				Map m = parseXmlToList2(notityXml.toString());
				WxPayResult wpr = new WxPayResult();
				wpr = setWxPayResultObj(wpr,m);
				// 判断微信返回的内容是否为SUCCESS
				assert "SUCCESS".equalsIgnoreCase(wpr.getResultCode());
				// 验证数据
				String orderIdMD5 = StaticInfo.orderIdMD5.decrypt(wpr.getAttach());
				assert orderIdMD5.equals(wpr.getOutTradeNo());
				// 组装数据
				Date nowDate = new Date();
				Order order = new Order();
				order.setSalesOrderid(orderIdMD5);
				order.setSalesStatus(2);
				order.setSalesTransactionId(wpr.getTransactionId());
				order.setSalesTransactionDate(nowDate);
				order.setSalesOpenid(wpr.getOpenid());
				Order or = adminOrderService.getShopOrderByOrderId(orderIdMD5);
				assert or.getSalesStatus() == 1;
				List<OrderItem> orderItemList = adminOrderService.getShopOrderItemByOrderId(or.getSalesOrderid());
				// 修改订单状态
				int count = cartService.updateOrderStatus(order);
				// 修改优惠券状态
				order=cartService.selectOrderInfoById(order);
				String couponId=order.getCouponId();
				if (couponId!=null && !"".equals(couponId)) {
					welfareService.useCoupon(wpr.getOpenid(), orderIdMD5, couponId);
				}

				//订单结算获取获取积分
				try{
					Integer configId = 2;		//日常 - 订单结算获取积分X*0.1
					Promotions promotions = activityUtil.getAllPromotions();
					if(promotions != null && promotions.getIntegralConfigList() !=null && promotions.getIntegralConfigList().size() > 0){
						List<IntegralConfig> integralConfigList = promotions.getIntegralConfigList();
						configId = integralConfigList.get(0).getInteConfId();
					}
					//获取积分
					Integer flag = integralLogService.addOrderChangIntegra(or.getSalesOrderid(),configId);
					if(flag < 1){
						ratFWLogger.info("订单获取积分失败，订单编号："+or.getSalesOrderid());
					}
				}catch (Exception e){
					ratFWLogger.info("error+订单获取积分失败，订单编号："+or.getSalesOrderid());
					e.printStackTrace();
				}

				assert count >= 1;
				// 给订单的回调修改状态
				{
					MQMessage message = new MQMessage();
					message.setServiceToMessage(orderIdMD5);
					message.setServiceOtherMessage(wpr.getTransactionId());
					message.setServiceIsSend(2);
					message.setServiceStatus(1);
					message.setServiceHandlerObjectName("cartServiceImpl");
					message.setServiceHandlerMethodName("supdateStateByOrderId");
					message.setServiceType(1);
					mQservice.insertMQMessage(message);
				}
				// 给RST推送信息
				{
					MQMessage message = new MQMessage();
					message.setServiceToMessage(orderIdMD5);
					message.setServiceOtherMessage("efast.trade.new.add");
					message.setServiceIsSend(2);
					message.setServiceStatus(1);
					message.setServiceType(1);
					message.setServiceHandlerObjectName("efastOrderServiceImpl");
					message.setServiceHandlerMethodName("addOrderInfoToEfast");
					mQservice.insertMQMessage(message);
				}
				//推送消息到微信
				{
					SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
					DecimalFormat df=new DecimalFormat("0.00");
					String payMoney = df.format((float)or.getSalesYetAmount()/100) + "元";
					sendMessageUtil.probationPaySuccess(
							ResultInfo.PAYSUCCESS,
							or.getSalesOrderid(),
							payMoney,
							"Aiberle微信商城",
							sdf.format(new Date()),
							ResultInfo.sendRemark1,
							order.getSalesOpenid(),
							ResultInfo.ORDER_DETAIL+order.getSalesOrderid());
				}
				BufferedOutputStream out = new BufferedOutputStream(response.getOutputStream());
				out.write(resXml.getBytes());
				out.flush();
				out.close();
			} catch (Exception e) {
				BufferedOutputStream out = new BufferedOutputStream(response.getOutputStream());
				out.write("<xml><return_code><![CDATA[FAIL]]></return_code><return_msg><![CDATA[报文有误]]></return_msg></xml>".getBytes());
				out.flush();
				out.close();
				LoggerFactory.getLogger(this.getClass()).error("", e);
			}
		}
	}
	
	
	/**
	 * 销售订单支付回调方法
	 */
	@SuppressWarnings("rawtypes")
	@RequestMapping("/drpPayBack")
	public void drpPayBack(HttpServletRequest request, 
			HttpServletResponse response) throws IOException {
		synchronized (lock) {
			String inputLine;
			String notityXml = "";
			String resXml = "<xml><return_code><![CDATA[SUCCESS]]></return_code><return_msg><![CDATA[OK]]></return_msg></xml>";
			try {
				//获取XML参数
				while ((inputLine = request.getReader().readLine()) != null) {
					notityXml += inputLine;
				}
				request.getReader().close();
				//解析XML格式的数据
				Map m = parseXmlToList2(notityXml);
				WxPayResult wpr = new WxPayResult();
				wpr=setWxPayResultObj(wpr,m);
				//判断
				if("SUCCESS".equalsIgnoreCase(wpr.getResultCode())){
					//resXml:返回给微信服务器的数据
					resXml = "<xml><return_code><![CDATA[SUCCESS]]></return_code><return_msg><![CDATA[OK]]></return_msg></xml>";
					String orderIdMD5 = StaticInfo.orderIdMD5.decrypt(wpr.getAttach());
					if (orderIdMD5.equals(wpr.getOutTradeNo())) {
						PlaceOrder placeOrder=new PlaceOrder();
						placeOrder.setId(orderIdMD5);
						placeOrder.setStatus(2); //已付款
						 int count = placeOrderService.UpdateOrder(placeOrder);
						if (count < 1) {
							resXml = "<xml><return_code><![CDATA[FAIL]]></return_code><return_msg><![CDATA[报文有误]]></return_msg></xml>";
						}
					} else {
						resXml = "<xml><return_code><![CDATA[FAIL]]></return_code><return_msg><![CDATA[报文有误]]></return_msg></xml>";
						ratFWLogger.error(this, "订单:" + orderIdMD5 + "支付成功，但是没有通过校验!");
					}
				}else{
					resXml = "<xml><return_code><![CDATA[FAIL]]></return_code><return_msg><![CDATA[报文为空]]></return_msg></xml>";
				}
				BufferedOutputStream out = new BufferedOutputStream(response.getOutputStream());
				out.write(resXml.getBytes());
				out.flush();
				out.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * 解析xml片段为Map
	 * @param xml
	 * @return
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private Map parseXmlToList2(String xml) {
		Map retMap = new HashMap();
		try {
			StringReader read = new StringReader(xml);
			// 创建新的输入源SAX 解析器将使用 InputSource 对象来确定如何读取 XML 输入
			InputSource source = new InputSource(read);
			// 创建一个新的SAXBuilder
			SAXBuilder sb = new SAXBuilder();
			// 防止XXE
			sb.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true);
			sb.setFeature("http://xml.org/sax/features/external-general-entities", false);
			sb.setFeature("http://xml.org/sax/features/external-parameter-entities", false);
			sb.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false);

			// 通过输入源构造一个Document
			Document doc = (Document) sb.build(source);
			Element root = doc.getRootElement();// 指向根节点
			List<Element> es = root.getChildren();
			if (es != null && es.size() != 0) {
				for (Element element : es) {
					retMap.put(element.getName(), element.getValue());
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return retMap;
	}
    
	//封装参数
	public WxPayResult setWxPayResultObj(WxPayResult wpr ,Map m){
		//解析参数
		wpr.setAppid(m.get("appid").toString());
		wpr.setBankType(m.get("bank_type").toString());
		wpr.setCashFee(m.get("cash_fee").toString());
		wpr.setFeeType(m.get("fee_type").toString());
		wpr.setIsSubscribe(m.get("is_subscribe").toString());
		wpr.setMchId(m.get("mch_id").toString());
		wpr.setNonceStr(m.get("nonce_str").toString());
		wpr.setOpenid(m.get("openid").toString());
		wpr.setOutTradeNo(m.get("out_trade_no").toString());
		wpr.setAttach(m.get("attach").toString());
		wpr.setResultCode(m.get("result_code").toString());
		wpr.setReturnCode(m.get("return_code").toString());
		wpr.setSign(m.get("sign").toString());
		wpr.setTimeEnd(m.get("time_end").toString());
		wpr.setTotalFee(m.get("total_fee").toString());
		wpr.setTradeType(m.get("trade_type").toString());
		wpr.setTransactionId(m.get("transaction_id").toString());
		return wpr;
	}

	/**
	 * 测试订单推送
	 * @param orderId
	 * @return
	 */
	@ResponseBody
	@RequestMapping(value = "/test_order")
	public ResultMsg checkOrderPaySuccess(
			@RequestParam(value = "orderId") String orderId) {
		ResultMsg msg = new ResultMsg();

		String orderIdMD5 = orderId;
		Order or = adminOrderService.getShopOrderByOrderId(orderIdMD5);

		// 组装数据
		Date nowDate = new Date();
		Order order = new Order();
		order.setSalesOrderid(orderIdMD5);
		order.setSalesStatus(2);
		order.setSalesTransactionId("xxxxxxxxx");
		order.setSalesTransactionDate(nowDate);
		order.setSalesOpenid(or.getSalesOpenid());

		assert or.getSalesStatus() == 1;
		List<OrderItem> orderItemList = adminOrderService.getShopOrderItemByOrderId(or.getSalesOrderid());
		// 修改订单状态
		int count = cartService.updateOrderStatus(order);
		// 修改优惠券状态
		order=cartService.selectOrderInfoById(order);
		String couponId=order.getCouponId();
		if (couponId!=null && !"".equals(couponId)) {
			welfareService.useCoupon(or.getSalesOpenid(), orderIdMD5, couponId);
		}
		assert count >= 1;

		//订单结算获取获取积分
		try{
			Integer configId = 2;		//日常 - 订单结算获取积分X*0.1
			Promotions promotions = activityUtil.getAllPromotions();
			if(promotions != null && promotions.getIntegralConfigList() !=null && promotions.getIntegralConfigList().size() > 0){
				List<IntegralConfig> integralConfigList = promotions.getIntegralConfigList();
				configId = integralConfigList.get(0).getInteConfId();
			}
			//获取积分
			Integer flag = integralLogService.addOrderChangIntegra(or.getSalesOrderid(),configId);
			if(flag < 1){
				ratFWLogger.info("订单获取积分失败，订单编号："+or.getSalesOrderid());
			}
		}catch (Exception e){
			ratFWLogger.info("error+订单获取积分失败，订单编号："+or.getSalesOrderid());
			e.printStackTrace();
		}

		//推送消息到微信
		{
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
			DecimalFormat df=new DecimalFormat("0.00");
			String payMoney = df.format((float)or.getSalesYetAmount()/100) + "元";
			sendMessageUtil.probationPaySuccess(
					ResultInfo.PAYSUCCESS,
					or.getSalesOrderid(),
					payMoney,
					"Aiberle微信商城",
					sdf.format(new Date()),
					ResultInfo.sendRemark1,
					order.getSalesOpenid(),
					ResultInfo.ORDER_DETAIL+order.getSalesOrderid());
		}
		return msg;
	}

	@ResponseBody
	@RequestMapping(value = "/test_accountEntry")
	public void accountEntry() throws Exception {
		ratFWLogger.info("---------------- 待入账转为已入账-开始 ---------------");
		List<IntegralLog> integralLogList = integralLogService.getIntegralLogListByWaitUser();
		if(integralLogList!=null && integralLogList.size()>0){
			ratFWLogger.info("---------------- 当前待入账的处理数量-"+ integralLogList.size() +" ---------------");
			for (IntegralLog integralLog:integralLogList) {
				try {
					integralLogService.waitUser(integralLog);
				}catch (Exception e){
					ratFWLogger.info("--待入账转为已入账失败；待入账积分记录id："+integralLog.getInteLogId()+"；" +
							"订单编号："+ integralLog.getInteLogWaitingOrderId() +";失败原因："+e.getMessage());
					e.printStackTrace();
				}
			}
			ratFWLogger.info("---------------- 待入账转为已入账-结束 ---------------");
		}
	}

	@ResponseBody
	@RequestMapping(value = "/test_birthDatePromptCoupon")
	public void birthDatePromptCoupon() throws Exception {
		ratFWLogger.info("---------------- 生日提醒-赠送优惠券-开始 ---------------");
		List<Member> memberList = memberService.getNowBirthDate();
		ratFWLogger.info("---今日生日的用户数量："+memberList.size());
		Integer couponId = 40000;
		for (Member member:memberList) {
			CouponType couponType = couponTypeService.getCouponTypeById(couponId);

			Calendar calendar = Calendar.getInstance();
			calendar.set(Calendar.HOUR_OF_DAY,23);
			calendar.set(Calendar.MINUTE,59);
			calendar.set(Calendar.SECOND,50);
			//创建优惠券
			CouponItem couponItem = new CouponItem();
			String uuidStr = StrUtils.getUUID();
			couponItem.setCouponItemId(uuidStr);
			couponItem.setCouponItemUseropenid(member.getUserOpenid());
			couponItem.setCouponId(couponType.getCouponId());
			couponItem.setCouponReceiveDate(new Date());
			couponItem.setCouponUseEndDate(calendar.getTime());
			couponItem.setCouponUseStatus(1);
			couponItem.setCouponItemRemark(couponType.getCouponRemark());
			Integer flag = couponItemService.insertCouponItem(couponItem);
			if(flag < 1){
				ratFWLogger.info("---生日优惠券领取失败，userOpenId："+member.getUserOpenid() + "");
			}else{
				ratFWLogger.info("---生日优惠券领取成功，userOpenId："+member.getUserOpenid() + "");
			}
		}
		ratFWLogger.info("---------------- 生日提醒-赠送优惠券-结束 ---------------");
	}

	@ResponseBody
	@RequestMapping(value = "/test_birthDatePrompt")
	public void birthDatePrompt() throws Exception {
		ratFWLogger.info("---------------- 生日提醒-提醒用户-开始 ---------------");
		List<Member> memberList = memberService.getNowBirthDate();
		ratFWLogger.info("---今日生日的用户数量："+memberList.size());
		for (Member member:memberList) {
			//推送微信模板消息
			try {
				sendMessageUtil.memberActivation(
						"尊敬的会员，祝您生日快乐，特送上一份生日礼包。",
						member.getUserTel(),
						String.valueOf(member.getUserSurplusIntegral()),
						"1、免费领取50元代金券，满99元即可使用，在会员中心-券包即可查看使用。\\\\r\\\\n" +
								"2、会员生日当天下单购买产品，即可享受3倍积分。" +
								"生日福利仅限生日当天领取和使用，赶快享受你的专属生日礼包吧！",
						member.getUserOpenid(),
						ResultInfo.COUPON_URL);
			}catch (Exception e){
				ratFWLogger.info("推送今日生日的用户消息失败，失败openId：" + member.getUserOpenid());
				e.printStackTrace();
			}
		}
		ratFWLogger.info("---------------- 生日提醒-提醒用户-结束 ---------------");
	}

	@ResponseBody
	@RequestMapping(value = "/register_member")
	public void registerMember() throws Exception {
		ratFWLogger.info("---------------- 关注用户过一个月还未注册，推送注册信息-开始 ---------------");
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(new Date());
		calendar.add(Calendar.DATE, -30);//30天前

		Member member = new Member();
		member.setUserSubDate(calendar.getTime());
		List<Member> memberList = memberService.getNoRegister(member);

		if(memberList!=null && memberList.size()>0){
			ratFWLogger.info("---------------- 当前任务处理数量-"+ memberList.size() +" ---------------");
			for (Member m:memberList) {
				try {
					sendMessageUtil.registerMember(
							"欢迎关注Aiberle爱贝源公众号。现在注册会员可赠送100积分，享受会员折扣，购买滤芯产品88折，非滤芯产品95折优惠，还赠送单支滤芯买一送一优惠券！",
							"我的爱贝源-会员中心-成为会员",
							"400-8006692",
							"赶紧注册会员吧！更多惊喜等你来！",
							m.getUserOpenid(),
							ResultInfo.HUIYUAN_URL);
				}catch (Exception e){
					ratFWLogger.info("推送今日生日的用户消息失败，失败openId：" + member.getUserOpenid());
					e.printStackTrace();
				}
			}
			ratFWLogger.info("---------------- 关注用户过一个月还未注册，推送注册信息-结束 ---------------");
		}
	}

	@ResponseBody
	@RequestMapping(value = "/mq_recommend_use_coupon")
	public void mqRecommendUseCoupon() throws Exception {
		ratFWLogger.info("---------------- 注册会员未使用优惠券提醒-开始 ---------------");

		//45天未使用优惠券的券
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(new Date());
		calendar.add(Calendar.DATE, -45);//45天前
		CouponItem couponItem = new CouponItem();
		couponItem.setCouponId(40001);
		couponItem.setCouponReceiveDate(calendar.getTime());
		List<CouponItemDto> couponItemDtoList = couponItemService.getNoUseCouponList(couponItem);
		if(couponItemDtoList!=null && couponItemDtoList.size()>0){
			ratFWLogger.info("---------------- 45天未使用优惠券的券 - 前当前任务处理数量-"+ couponItemDtoList.size() +" ---------------");
			for (CouponItemDto couponItemDto:couponItemDtoList) {
				try {
					sendMessageUtil.recommendUseCoupon(
							"亲爱的用户，您的单支滤芯买一送一优惠券有效期还有45天",
							"单支滤芯买一送一优惠券",
							"45天后失效",
							"可在“服务中心-会员中心-券包”查看，请尽快使用。",
							couponItemDto.getUseropenid(),
							ResultInfo.COUPON_URL);
				}catch (Exception e){
					ratFWLogger.info("注册成功提醒使用优惠券失败，失败openId：" + couponItemDto.getUseropenid()+",优惠券ID："+couponItemDto.getCouponItemId());
					e.printStackTrace();
				}
			}
			ratFWLogger.info("---------------- 注册会员未使用优惠券提醒-结束 ---------------");
		}

		//85天未使用优惠券的券
		Calendar nowcalendar = Calendar.getInstance();
		nowcalendar.setTime(new Date());
		nowcalendar.add(Calendar.DATE, -85);//45天前
		CouponItem nowcouponItem = new CouponItem();
		nowcouponItem.setCouponId(40001);
		nowcouponItem.setCouponReceiveDate(nowcalendar.getTime());
		List<CouponItemDto> nowcouponItemDtoList = couponItemService.getNoUseCouponList(nowcouponItem);
		if(nowcouponItemDtoList!=null && nowcouponItemDtoList.size()>0){
			ratFWLogger.info("---------------- 85天未使用优惠券的券 - 前当前任务处理数量-"+ nowcouponItemDtoList.size() +" ---------------");
			for (CouponItemDto couponItemDto:nowcouponItemDtoList) {
				try {
					sendMessageUtil.recommendUseCoupon(
							"亲爱的用户，您的单支滤芯买一送一优惠券即将到期",
							"单支滤芯买一送一优惠券",
							"5天后失效",
							"可在“服务中心-会员中心-券包”查看，请尽快使用。",
							couponItemDto.getUseropenid(),
							ResultInfo.COUPON_URL);
				}catch (Exception e){
					ratFWLogger.info("注册成功提醒使用优惠券失败，失败openId：" + couponItemDto.getUseropenid()+",优惠券ID："+couponItemDto.getCouponItemId());
					e.printStackTrace();
				}
			}
		}
		ratFWLogger.info("---------------- 注册会员未使用优惠券提醒-结束 ---------------");
	}




}
